/******************************************************************************
 *
 *  Project  :   Mocca, BLINKY
 *  Author   :   La
 *  Date     :   
 *  Version  :   
 *  File     :   AVR_Treiebr.C
 *
******************************************************************************/
           
/******************************************************************************
///////////////////////////////////////////////////////////////////////////////
                            USER CONFIGURATION
                        EDIT THE FOLLOWING SECTION
///////////////////////////////////////////////////////////////////////////////
******************************************************************************/

// Example/template of a main-program for:
//         -  Compiler and IDE:  ATMEL, AtmelStudio
//         -  Microcontroller:   Mocca
/*****************************************************************************/

#include <avr/io.h>
#include "os_minios.h"

void lcd_sid(uint8_t status);
void lcd_sclk(uint8_t status);
void write_lcd_f(uint16_t rs, uint16_t value);

volatile uint16_t takt_10ms_zaehler;


//--------------------------------------------------------------------------------------------
// Initialisirung SiriusBlack
//--------------------------------------------------------------------------------------------

void init_sirius(void)
{
	init_mocca();
}

void init_mocca(void)
{
	//PORTS definieren
	DDRA = 0xFF;	// LED Port als Ausgang
	DDRK = 0x00;	// Schalter Port als Eingang
	DDRE = 0x00;	// Taster als Eingang
	DDRG = 0x27;	// LCD port als Ausgang
	DDRH = 0x78;	// RGB, CTS als Ausgang
	DDRJ = 0xFF;	// Port J als Ausgang
	
	SysTimerStart();//Start des Systemtimers
	RTC_Init();		//RTC Initialisieren
	sei();			//Global Interrups aktivieren
}

//////////////////////////////////////////////////////////////////////////////////////////////
//                                   RTOS INITIALISIERUNG                                   //
//////////////////////////////////////////////////////////////////////////////////////////////
//--------------------------------------------------------------------------------------------
// Timer 0 Initialisierung
//--------------------------------------------------------------------------------------------
void SysTimerStart(void)
{
	TCCR0A = 0b00000011; // Timer mode Einstellungen -> Fast PWM mode (Einstellbarer Max Wert)
	TCCR0B = 0b00001101; // clk / 128 (Prescaler)
	TIMSK0 = 0b00000001; // Timer overflow Interrupt aktivieren
	OCR0A  = 156;		 // 10ms Takt
}

//--------------------------------------------------------------------------------------------
// Timer 0 Overflow Interrupt ->  Wird alle 10ms aufgerufen
//--------------------------------------------------------------------------------------------
ISR(TIMER0_OVF_vect)
{
	OS_DISABLE_INT		//Alle Interrupts deaktivieren
	takt_10ms_zaehler --;
	SysTimerStart();
	OS_TimerTick();	    //Produces Systemclock
	OS_ENABLE_INT		//Alle Interrupts aktivieren
}

//--------------------------------------------------------------------------------------------
// RTC initialisierung
//--------------------------------------------------------------------------------------------
void RTC_Init(void)
{
	ASSR   = 0b00100000;	//Externer Clock selected (Quarz)
	TCCR2A = 0b00000000;	//Normal Mode
	TCCR2B = 0b00000101;	//Prescaler 128 (32.768kHz / 128) => 1s Takt
	OCR2A  = 255;			//Maximaler Zlerwert
	TIMSK2 = 0b00000001;	//Flag Enable
}

//--------------------------------------------------------------------------------------------
// RTC Overflow Interrupt
//--------------------------------------------------------------------------------------------
ISR(TIMER2_OVF_vect)
{
	OS_DISABLE_INT			//Alle Interrupts ausschalten
	OS_SignalSem_ISR(1);	//Signal Nummer Eins ist eingetroffen (1s ist vorbei)
	OS_ENABLE_INT			//Alle Interrupts aktivieren
}
//////////////////////////////////////////////////////////////////////////////////////////////
//                                   RTOS INITIALISIERUNG                                   //
//////////////////////////////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------------------------------
// 10ms Wartefunktion
//--------------------------------------------------------------------------------------------
void wait_10ms(uint16_t faktor)
{
	if(faktor > 1) faktor = faktor / 2;
	takt_10ms_zaehler = faktor;
	if(faktor != 0) while(takt_10ms_zaehler);
}

//--------------------------------------------------------------------------------------------
// 1us Wartefunktion
//--------------------------------------------------------------------------------------------
void wait_1us(uint16_t volatile faktor)
{
	while (faktor)
	{	
		// 11 Taktzklen keine operation , restliche 5 taktzyklen fr while und faktor increment
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		asm  ("nop");
		faktor --;
	}
}

